/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_MAGNETOMETRIC_H
#define _GCTL_MAGNETOMETRIC_H

#include "cstddef"

namespace gctl
{   
    /**
     * @brief    Forward modeling of the magnetometric resistivity (MMR) data of a outcropping hemispherical depression
     * 
     * @note     Edwards et al., (1978). On the theory of magnetometric resistivity (MMR) methods. Geophysics, 43(6), 1176-1203.
     * 
     * @param x  Field point coordinate x
     * @param y  Field point coordinate y
     * @param cx Current source coordinate x
     * @param cy Current source coordinate y
     * @param I  Current value
     * @param hx Center coordinate x of the depression
     * @param hy Center coordinate y of the depression
     * @param hr Radius of the depression
     * @param cndt conductivity of the depression
     * @param bkg_cndt conductivity of the host medium
     * @param order Calculating order of the legendre polynomials (default is 10)
     * @return Vertical component (Bz) of the anomalous magnetic field
     */
    double mmr_hemisphere_bz(double x, double y, double cx, double cy, double I, 
        double hx, double hy, double hr, double cndt, double bkg_cndt, size_t order = 10);

} // namespace gctl

#endif // _GCTL_MAGNETOMETRIC_H